@@ -69,11 +69,6 @@ e.g. database.schema.table is represented as ("table", "schema", "database")
 QualifiedName = tuple[str, ...]
 
 
-@dataclass(frozen=True, kw_only=True, slots=True)
-class Alias:
-    name: QualifiedName
-
-
 @dataclass(frozen=True, kw_only=True, slots=True)
 class Type:
     name: str
@@ -128,6 +123,7 @@ class Uniqueness:
     on_conflict: OnConflict | None = None
 
     def columns(self, /) -> Iterable[str]:
+        """Names of indexed columns"""
         return (x.column for x in self.indexed)
 
 
@@ -136,8 +132,8 @@ class ForeignKey:
     name: str | None = None
     is_table_constraint: bool = False
     columns: tuple[str, ...]
-    foreign_table: Alias
-    referred_columns: tuple[str, ...] | None
+    foreign_table: QualifiedName
+    referred_columns: tuple[str, ...] | None = None
     on_delete: OnUpdateDelete | None = None
     on_update: OnUpdateDelete | None = None
     match: Match | None = None
@@ -157,11 +153,31 @@ TableConstraint = Uniqueness | ForeignKey | Check
 ColumnConstraint = TableConstraint | Collation | Default | NotNull | Generated
 
 
+def uniqueness(constraints: Iterable[ColumnConstraint], /) -> Iterable[Uniqueness]:
+    """All uniqueness constraints, including the primary key"""
+    return (x for x in constraints if isinstance(x, Uniqueness))
+
+
+def primary_key(constraints: Iterable[ColumnConstraint], /) -> Uniqueness | None:
+    """First found primary key constraint"""
+    return next((x for x in uniqueness(constraints) if x.is_primary), None)
+
+
+def foreign_keys(constraints: Iterable[ColumnConstraint], /) -> Iterable[ForeignKey]:
+    """All foreign key constraints"""
+    return (x for x in constraints if isinstance(x, ForeignKey))
+
+
+def checks(constraints: Iterable[ColumnConstraint], /) -> Iterable[Check]:
+    """All foreign key constraints"""
+    return (x for x in constraints if isinstance(x, Check))
+
+
 @dataclass(frozen=True, kw_only=True, slots=True)
 class Column:
     name: str
-    type: Type
-    constraints: tuple[ColumnConstraint, ...]
+    type: Type = Type(name="")
+    constraints: tuple[ColumnConstraint, ...] = tuple()
 
     def collation(self, /) -> Collation | None:
         return next((x for x in self.constraints if isinstance(x, Collation)), None)
@@ -195,13 +211,23 @@ class TableOptions:
 class Table:
     name: QualifiedName
     columns: tuple[Column, ...]
-    constraints: tuple[TableConstraint, ...]
+    constraints: tuple[TableConstraint, ...] = tuple()
     options: TableOptions = TableOptions()
     if_not_exists: bool = False
     or_replace: bool = False
     temporary: bool = False
 
+    def column(self, name: str, /) -> Column | None:
+        return next(x for x in self.columns if x.name == name)
+
+    def generated_columns(self, /) -> Iterable[Column]:
+        return (x for x in self.columns if x.generated() is not None)
+
+    def non_generated_columns(self, /) -> Iterable[Column]:
+        return (x for x in self.columns if x.generated() is None)
+
     def all_constraints(self, /) -> Iterable[TableConstraint]:
+        """Constraints declared on the table and on the columns"""
         return itertools.chain(
             itertools.chain.from_iterable(
                 col.table_constraints() for col in self.columns
@@ -211,35 +237,84 @@ class Table:
 
     def primary_key(self, /) -> Uniqueness | None:
         """First found primary key constraint"""
-        return next(
-            (
-                x
-                for x in self.all_constraints()
-                if isinstance(x, Uniqueness) and x.is_primary
-            ),
-            None,
-        )
+        return primary_key(self.all_constraints())
 
     def uniqueness(self, /) -> Iterable[Uniqueness]:
         """All uniqueness constraints, including the primary key"""
-        return (x for x in self.all_constraints() if isinstance(x, Uniqueness))
+        return uniqueness(self.all_constraints())
 
     def foreign_keys(self, /) -> Iterable[ForeignKey]:
         """All foreign key constraints"""
-        return (x for x in self.all_constraints() if isinstance(x, ForeignKey))
+        return foreign_keys(self.all_constraints())
 
     def checks(self, /) -> Iterable[Check]:
         """All foreign key constraints"""
-        return (x for x in self.all_constraints() if isinstance(x, Check))
+        return checks(self.all_constraints())
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
+class Index:
+    name: QualifiedName
+    table: str
+    indexed: tuple[Indexed, ...]
+    where: tuple[tok.Token, ...] | None
+    if_not_exists: bool = False
+    unique: bool = False
+
+
+SchemaItem = Index | Table
 
 
 @dataclass(frozen=True, kw_only=True, slots=True)
 class Schema:
-    tables: tuple[Table, ...]
+    items: tuple[SchemaItem, ...]
+
+    def tables(self, /) -> Iterable[Table]:
+        return (x for x in self.items if isinstance(x, Table))
+
+    def indexes(self, /) -> Iterable[Index]:
+        return (x for x in self.items if isinstance(x, Index))
+
+    def unique_indexes(self, /) -> Iterable[Index]:
+        return (x for x in self.indexes() if x.unique)
 
 
 Symbols = dict[str, Table]
 
 
 def symbols(schema: Schema, /) -> Symbols:
-    return {tbl.name[0]: tbl for tbl in schema.tables}
+    return {tbl.name[0]: tbl for tbl in schema.tables()}
+
+
+def referred_columns(fk: ForeignKey, symbols: Symbols, /) -> tuple[str, ...]:
+    """referred columns of `fk` or primary key of the foreign table"""
+    foreign_table = symbols.get(fk.foreign_table[0])
+    assert (
+        foreign_table is not None
+    ), f"Table '{fk.foreign_table[0]}' is not present in `symbols`"
+    referred_columns = fk.referred_columns
+    if referred_columns is None:
+        f_pk = foreign_table.primary_key()
+        assert f_pk is not None
+        return tuple(f_pk.columns())
+    else:
+        return referred_columns
+
+
+def resolve_foreign_key(
+    fk: ForeignKey, col: str, symbols: Symbols
+) -> Iterable[ForeignKey | str]:
+    assert col in fk.columns
+    foreign_table = symbols.get(fk.foreign_table[0])
+    assert (
+        foreign_table is not None
+    ), f"Table '{fk.foreign_table[0]}' is not present in `symbols`"
+    ref_cols = referred_columns(fk, symbols)
+    assert len(fk.columns) == len(ref_cols)
+    f_col = ref_cols[fk.columns.index(col)]
+    for f_fk in foreign_table.foreign_keys():
+        if f_col in f_fk.columns:
+            yield f_fk
+            yield from resolve_foreign_key(f_fk, f_col, symbols)
+            return
+    yield f_col
